/*!
** @file
** @brief   The enumeration of display monitors and top-level windows.
** @remarks It makes a callback to the member function of the derived class.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
** @license The MIT License
** @verbatim

Copyright (C) 2014 Ueta System Design Studio

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@endverbatim
*/
#pragma once

#pragma warning(push)
#pragma warning(disable: 4100)

/*!
** @brief   The enumeration of display monitors.
** @remarks -
*/
class ATL_NO_VTABLE CEnumDisplayMonitors
{
public:

	/*!
	** @brief   Enumerates all display monitors on the desktop.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL EnumDisplayMonitors()
	{
		return ::EnumDisplayMonitors(NULL, NULL, &CEnumDisplayMonitors::MonitorEnumProc, LPARAM(this));
	}

	/*!
	** @brief   The callback function that is called by the EnumDisplayMonitors() function.
	** @param   [in] hMonitor a handle to the display monitor.
	** @returns TRUE to continue the enumeration, or FALSE otherwise.
	** @remarks -
	*/
	virtual BOOL EnumDisplayMonitorsProc(HMONITOR hMonitor) = 0;

private:

	static BOOL CALLBACK MonitorEnumProc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
	{
		CEnumDisplayMonitors* that = reinterpret_cast<CEnumDisplayMonitors*>(dwData);

		return that->EnumDisplayMonitorsProc(hMonitor);
	}
};

/*!
** @brief   The enumeration of top-level windows.
** @remarks -
*/
class ATL_NO_VTABLE CEnumTopLevelWindows
{
public:

	/*!
	** @brief   Enumerates all top-level windows on the screen.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL EnumTopLevelWindows()
	{
		return ::EnumWindows(&CEnumTopLevelWindows::EnumWindowsProc, LPARAM(this));
	}

	/*!
	** @brief   The callback function that is called by the EnumTopLevelWindows() function.
	** @param   [in] hWnd a handle to the top-level windows.
	** @returns TRUE to continue the enumeration, or FALSE otherwise.
	** @remarks -
	*/
	virtual BOOL EnumTopLevelWindowsProc(HWND hWnd) = 0;

private:

	static BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam)
	{
		CEnumTopLevelWindows* that = reinterpret_cast<CEnumTopLevelWindows*>(lParam);

		return that->EnumTopLevelWindowsProc(hWnd);
	}
};

#pragma warning(pop)
